@@ -8,18 +8,18 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
     B<-data[,2][data[,1]=="B"]
     
     if(CL=="mean"){
-      CLA<-mean(A)	
-      CLB<-mean(B)
+      CLA<-mean(A,na.rm=TRUE)	
+      CLB<-mean(B,na.rm=TRUE)
     }
     if(CL=="median"){
-      CLA<-median(A)	
-      CLB<-median(B)
+      CLA<-median(A,na.rm=TRUE)	
+      CLB<-median(B,na.rm=TRUE)
     }
     if(CL=="bmed"){
       aa<-sort(A)
       bb<-sort(B)
       if(length(aa)<5){
-        CLA<-median(A)
+        CLA<-median(A,na.rm=TRUE)
       }
       if(length(aa)==5|length(aa)==7|length(aa)==9|length(aa)==11){
         CLA<-(aa[ceiling(length(aa)/2)-1]+aa[ceiling(length(aa)/2)]+aa[ceiling(length(aa)/2)+1])/3
@@ -34,7 +34,7 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
         CLA<-1/10*aa[length(aa)/2-2]+1/5*aa[length(aa)/2-1]+1/5*aa[length(aa)/2]+1/5*aa[length(aa)/2+1]+1/5*aa[length(aa)/2+2]+1/10*aa[length(aa)/2+3]
       }
       if(length(bb)<5){
-        CLB<-median(B)
+        CLB<-median(B,na.rm=TRUE)
       }
       if(length(bb)==5|length(bb)==7|length(bb)==9|length(bb)==11){
         CLB<-(bb[ceiling(length(bb)/2)-1]+bb[ceiling(length(bb)/2)]+bb[ceiling(length(bb)/2)+1])/3
@@ -50,16 +50,17 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
       }
     }
     if(CL=="trimmean"){
-      CLA<-mean(A,trim=tr)	
-      CLB<-mean(B,trim=tr)
+      CLA<-mean(A,trim=tr,na.rm=TRUE)	
+      CLB<-mean(B,trim=tr,na.rm=TRUE)
     }  
     if(CL=="mest"){
       hpsi<-function(x,bend=1.28){
         hpsi<-ifelse(abs(x)<=bend,x,bend*sign(x))
         hpsi
       }
-      mest<-function(x,bend=1.28,na.rm=F){
-        if(na.rm)x<-x[!is.na(x)]
+      mest<-function(x,bend=1.28,na.rm=TRUE){
+        if(na.rm) x<-x[!is.na(x)]
+        if(length(x)==0) return(NA)
         if(mad(x)==0)stop("MAD=0. The M-estimator cannot be computed.")
         y<-(x-median(x))/mad(x)
         A<-sum(hpsi(y,bend))
@@ -105,7 +106,7 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
       plot(x,data[,2],xlab=xlab,ylab=ylab,ylim=ylim,pch=16)
       lines(x[data[,1]=="A"],data[,2][data[,1]=="A"])
       lines(x[data[,1]=="B"],data[,2][data[,1]=="B"])
-      lines(c(sum(data[,1]=="A")+0.5,sum(data[,1]=="A")+0.5),c(min(data[,2])-5,max(data[,2])+5),lty=2)
+      lines(c(sum(data[,1]=="A")+0.5,sum(data[,1]=="A")+0.5),c(min(data[,2],ylim[1],na.rm=TRUE)-5,max(data[,2],ylim[2],na.rm=TRUE)+5),lty=2)
       mtext(labels[1],side=3,at=(sum(data[,1]=="A")+1)/2)
       mtext(labels[2],side=3,at=(sum(data[,1]=="A")+(sum(data[,1]=="B")+1)/2))
       lines(c(1,(sum(data[,1]=="A"))),c(CLA,CLA),lty=3)
@@ -120,16 +121,16 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
     B2<-data[,2][data[,1]=="B2"]
     
     if(CL=="mean"){
-      CLA1<-mean(A1)
-      CLB1<-mean(B1)
-      CLA2<-mean(A2)
-      CLB2<-mean(B2)
+      CLA1<-mean(A1,na.rm=TRUE)
+      CLB1<-mean(B1,na.rm=TRUE)
+      CLA2<-mean(A2,na.rm=TRUE)
+      CLB2<-mean(B2,na.rm=TRUE)
     }
     if(CL=="median"){
-      CLA1<-median(A1)
-      CLB1<-median(B1)
-      CLA2<-median(A2)
-      CLB2<-median(B2)
+      CLA1<-median(A1,na.rm=TRUE)
+      CLB1<-median(B1,na.rm=TRUE)
+      CLA2<-median(A2,na.rm=TRUE)
+      CLB2<-median(B2,na.rm=TRUE)
     }
     if(CL=="bmed"){
       aa1<-sort(A1)
@@ -137,7 +138,7 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
       aa2<-sort(A2)
       bb2<-sort(B2)
       if(length(aa1)<5){
-        CLA1<-median(data[,2][data[,1]=="A1"])
+        CLA1<-median(data[,2][data[,1]=="A1"],na.rm=TRUE)
       }
       if(length(aa1)==5|length(aa1)==7|length(aa1)==9|length(aa1)==11){
         CLA1<-(aa1[ceiling(length(aa1)/2)-1]+aa1[ceiling(length(aa1)/2)]+aa1[ceiling(length(aa1)/2)+1])/3
@@ -152,7 +153,7 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
         CLA1<-1/10*aa1[length(aa1)/2-2]+1/5*aa1[length(aa1)/2-1]+1/5*aa1[length(aa1)/2]+1/5*aa1[length(aa1)/2+1]+1/5*aa1[length(aa1)/2+2]+1/10*aa1[length(aa1)/2+3]
       }
       if(length(bb1)<5){
-        CLB1<-median(data[,2][data[,1]=="B1"])
+        CLB1<-median(data[,2][data[,1]=="B1"],na.rm=TRUE)
       }
       if(length(bb1)==5|length(bb1)==7|length(bb1)==9|length(bb1)==11){
         CLB1<-(bb1[ceiling(length(bb1)/2)-1]+bb1[ceiling(length(bb1)/2)]+bb1[ceiling(length(bb1)/2)+1])/3
@@ -167,7 +168,7 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
         CLB1<-1/10*bb1[length(bb1)/2-2]+1/5*bb1[length(bb1)/2-1]+1/5*bb1[length(bb1)/2]+1/5*bb1[length(bb1)/2+1]+1/5*bb1[length(bb1)/2+2]+1/10*bb1[length(bb1)/2+3]
       }
       if(length(aa2)<5){
-        CLA2<-median(data[,2][data[,1]=="A2"])
+        CLA2<-median(data[,2][data[,1]=="A2"],na.rm=TRUE)
       }
       if(length(aa2)==5|length(aa2)==7|length(aa2)==9|length(aa2)==11){
         CLA2<-(aa2[ceiling(length(aa2)/2)-1]+aa2[ceiling(length(aa2)/2)]+aa2[ceiling(length(aa2)/2)+1])/3
@@ -182,7 +183,7 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
         CLA2<-1/10*aa2[length(aa2)/2-2]+1/5*aa2[length(aa2)/2-1]+1/5*aa2[length(aa2)/2]+1/5*aa2[length(aa2)/2+1]+1/5*aa2[length(aa2)/2+2]+1/10*aa2[length(aa2)/2+3]
       }
       if(length(bb2)<5){
-        CLB2<-median(data[,2][data[,1]=="B2"])
+        CLB2<-median(data[,2][data[,1]=="B2"],na.rm=TRUE)
       }
       if(length(bb2)==5|length(bb2)==7|length(bb2)==9|length(bb2)==11){
         CLB2<-(bb2[ceiling(length(bb2)/2)-1]+bb2[ceiling(length(bb2)/2)]+bb2[ceiling(length(bb2)/2)+1])/3
@@ -198,18 +199,19 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
       }
     }
     if(CL=="trimmean"){
-      CLA1<-mean(A1,trim=tr)
-      CLB1<-mean(B1,trim=tr)
-      CLA2<-mean(A2,trim=tr)
-      CLB2<-mean(B2,trim=tr)
+      CLA1<-mean(A1,trim=tr,na.rm=TRUE)
+      CLB1<-mean(B1,trim=tr,na.rm=TRUE)
+      CLA2<-mean(A2,trim=tr,na.rm=TRUE)
+      CLB2<-mean(B2,trim=tr,na.rm=TRUE)
     }
     if(CL=="mest"){
       hpsi<-function(x,bend=1.28){
         hpsi<-ifelse(abs(x)<=bend,x,bend*sign(x))
         hpsi
       }
-      mest<-function(x,bend=1.28,na.rm=F){
+      mest<-function(x,bend=1.28,na.rm=TRUE){
         if(na.rm)x<-x[!is.na(x)]
+        if(length(x)==0) return(NA)
         if(mad(x)==0)stop("MAD=0. The M-estimator cannot be computed.")
         y<-(x-median(x))/mad(x)
         A<-sum(hpsi(y,bend))
@@ -234,8 +236,9 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
     lines(x[data[,1]=="A1"],data[,2][data[,1]=="A1"])
     lines(x[data[,1]=="B1"],data[,2][data[,1]=="B1"])
     lines(x[data[,1]=="A2"],data[,2][data[,1]=="A2"])
-    lines(c(sum(data[,1]=="A1")+0.5,sum(data[,1]=="A1")+0.5),c(min(data[,2])-5,max(data[,2])+5),lty=2)
-    lines(c(sum(data[,1]=="A1")+sum(data[,1]=="B1")+0.5,sum(data[,1]=="A1")+sum(data[,1]=="B1")+0.5),c(min(data[,2])-5,max(data[,2])+5),lty=2)
+    lines(c(sum(data[,1]=="A1")+0.5,sum(data[,1]=="A1")+0.5),c(min(data[,2],ylim[1],na.rm=TRUE)-5,max(data[,2],ylim[2],na.rm=TRUE)+5),lty=2)
+    lines(c(sum(data[,1]=="A1")+sum(data[,1]=="B1")+0.5,sum(data[,1]=="A1")+sum(data[,1]=="B1")+0.5),
+          c(min(data[,2],ylim[1],na.rm=TRUE)-5,max(data[,2],ylim[2],na.rm=TRUE)+5),lty=2)
     mtext(labels[1],side=3,at=(sum(data[,1]=="A1")+1)/2)
     mtext(labels[2],side=3,at=(sum(data[,1]=="A1")+(sum(data[,1]=="B1")+1)/2))
     mtext(labels[3],side=3,at=(sum(data[,1]=="A1")+sum(data[,1]=="B1")+(sum(data[,1]=="A2")+1)/2))
@@ -244,7 +247,8 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
     lines(c((sum(data[,1]=="A1")+sum(data[,1]=="B1")+1),(sum(data[,1]=="A1")+sum(data[,1]=="B1")+sum(data[,1]=="A2"))),c(CLA2,CLA2),lty=3)
     if(design=="ABAB"){
       lines(x[data[,1]=="B2"],data[,2][data[,1]=="B2"])
-      lines(c(sum(data[,1]=="A1")+sum(data[,1]=="B1")+sum(data[,1]=="A2")+0.5,sum(data[,1]=="A1")+sum(data[,1]=="B1")+sum(data[,1]=="A2")+0.5),c(min(data[,2])-5,max(data[,2])+5),lty=2)
+      lines(c(sum(data[,1]=="A1")+sum(data[,1]=="B1")+sum(data[,1]=="A2")+0.5,sum(data[,1]=="A1")+sum(data[,1]=="B1")+sum(data[,1]=="A2")+0.5),
+            c(min(data[,2],ylim[1],na.rm=TRUE)-5,max(data[,2],ylim[2],na.rm=TRUE)+5),lty=2)
       mtext(labels[4],side=3,at=(sum(data[,1]=="A1")+sum(data[,1]=="B1")+sum(data[,1]=="A2")+(sum(data[,1]=="B2")+1)/2))
       lines(c((sum(data[,1]=="A1")+sum(data[,1]=="B1")+sum(data[,1]=="A2")+1),nrow(data)),c(CLB2,CLB2),lty=3)
     }
@@ -258,18 +262,18 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
       B<-data[,it*2][data[,(it*2)-1]=="B"]
       
       if(CL=="mean"){
-        CLA<-mean(A)	
-        CLB<-mean(B)
+        CLA<-mean(A,na.rm=TRUE)	
+        CLB<-mean(B,na.rm=TRUE)
       }
       if(CL=="median"){
-        CLA<-median(A)	
-        CLB<-median(B)
+        CLA<-median(A,na.rm=TRUE)	
+        CLB<-median(B,na.rm=TRUE)
       }
       if(CL=="bmed"){
         aa<-sort(A)
         bb<-sort(B)
         if(length(aa)<5){
-          CLA<-median(A)
+          CLA<-median(A,na.rm=TRUE)
         }
         if(length(aa)==5|length(aa)==7|length(aa)==9|length(aa)==11){
           CLA<-(aa[ceiling(length(aa)/2)-1]+aa[ceiling(length(aa)/2)]+aa[ceiling(length(aa)/2)+1])/3
@@ -284,7 +288,7 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
           CLA<-1/10*aa[length(aa)/2-2]+1/5*aa[length(aa)/2-1]+1/5*aa[length(aa)/2]+1/5*aa[length(aa)/2+1]+1/5*aa[length(aa)/2+2]+1/10*aa[length(aa)/2+3]
         }
         if(length(bb)<5){
-          CLB<-median(B)
+          CLB<-median(B,na.rm=TRUE)
         }
         if(length(bb)==5|length(bb)==7|length(bb)==9|length(bb)==11){
           CLB<-(bb[ceiling(length(bb)/2)-1]+bb[ceiling(length(bb)/2)]+bb[ceiling(length(bb)/2)+1])/3
@@ -300,16 +304,17 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
         }
       }  
       if(CL=="trimmean"){
-        CLA<-mean(A,trim=tr)	
-        CLB<-mean(B,trim=tr)
+        CLA<-mean(A,trim=tr,na.rm=TRUE)	
+        CLB<-mean(B,trim=tr,na.rm=TRUE)
       }
       if(CL=="mest"){
         hpsi<-function(x,bend=1.28){
           hpsi<-ifelse(abs(x)<=bend,x,bend*sign(x))
           hpsi
         }
-        mest<-function(x,bend=1.28,na.rm=F){
+        mest<-function(x,bend=1.28,na.rm=TRUE){
           if(na.rm)x<-x[!is.na(x)]
+          if(length(x)==0) return(NA)
           if(mad(x)==0)stop("MAD=0. The M-estimator cannot be computed.")
           y<-(x-median(x))/mad(x)
           A<-sum(hpsi(y,bend))
@@ -331,7 +336,8 @@ function(design,CL,tr,data=read.table(file.choose(new=FALSE)),xlab="Measurement
       plot(x,data[,it*2],xlab="",ylab=ylab,ylim=ylim,pch=16)
       lines(x[data[,(it*2)-1]=="A"],data[,it*2][data[,(it*2)-1]=="A"])
       lines(x[data[,(it*2)-1]=="B"],data[,it*2][data[,(it*2)-1]=="B"])
-      lines(c(sum(data[,(it*2)-1]=="A")+0.5,sum(data[,(it*2)-1]=="A")+0.5),c(min(data[,it*2])-5,max(data[,it*2])+5),lty=2)
+      lines(c(sum(data[,(it*2)-1]=="A")+0.5,sum(data[,(it*2)-1]=="A")+0.5),
+            c(min(data[,it*2],ylim[1],na.rm=TRUE)-5,max(data[,it*2],ylim[2],na.rm=TRUE)+5),lty=2)
       mtext(labels[1],side=3,at=(sum(data[,(it*2)-1]=="A")+1)/2)
       mtext(labels[2],side=3,at=(sum(data[,(it*2)-1]=="A")+(sum(data[,(it*2)-1]=="B")+1)/2))
       lines(c(1,(sum(data[,(it*2)-1]=="A"))),c(CLA,CLA),lty=3)
